@@ -24,20 +24,23 @@ RESERVED_ATTRS: Tuple[str, ...] = (
     'processName', 'relativeCreated', 'stack_info', 'thread', 'threadName')
 
 
-def merge_record_extra(record: logging.LogRecord, target: Dict, reserved: Union[Dict, List]) -> Dict:
+
+def merge_record_extra(record: logging.LogRecord, target: Dict, reserved: Union[Dict, List], rename_fields: Dict[str,str]) -> Dict:
     """
     Merges extra attributes from LogRecord object into target dictionary
 
     :param record: logging.LogRecord
     :param target: dict to update
     :param reserved: dict or list with reserved keys to skip
+    :param rename_fields: an optional dict, used to rename field names in the output.
+            Rename levelname to log.level: {'levelname': 'log.level'}
     """
     for key, value in record.__dict__.items():
         # this allows to have numeric keys
         if (key not in reserved
             and not (hasattr(key, "startswith")
                      and key.startswith('_'))):
-            target[key] = value
+            target[rename_fields.get(key,key)] = value
     return target
 
 
@@ -146,7 +149,7 @@ class JsonFormatter(logging.Formatter):
 
         This method is responsible for returning a list of fields (as strings)
         to include in all log messages.
-        """        
+        """
         if isinstance(self._style, logging.StringTemplateStyle):
             formatter_style_pattern = re.compile(r'\$\{(.+?)\}', re.IGNORECASE)
         elif isinstance(self._style, logging.StrFormatStyle):
@@ -154,7 +157,7 @@ class JsonFormatter(logging.Formatter):
         # PercentStyle is parent class of StringTemplateStyle and StrFormatStyle so
         # it needs to be checked last.
         elif isinstance(self._style, logging.PercentStyle):
-            formatter_style_pattern = re.compile(r'%\((.+?)\)s', re.IGNORECASE)
+            formatter_style_pattern = re.compile(r'%\((.+?)\)', re.IGNORECASE)
         else:
             raise ValueError('Invalid format: %s' % self._fmt)
 
@@ -168,18 +171,23 @@ class JsonFormatter(logging.Formatter):
         Override this method to implement custom logic for adding fields.
         """
         for field in self._required_fields:
-            if field in self.rename_fields:
-                log_record[self.rename_fields[field]] = record.__dict__.get(field)
-            else:
-                log_record[field] = record.__dict__.get(field)
+            log_record[field] = record.__dict__.get(field)
+
         log_record.update(self.static_fields)
         log_record.update(message_dict)
-        merge_record_extra(record, log_record, reserved=self._skip_fields)
+        merge_record_extra(record, log_record, reserved=self._skip_fields, rename_fields=self.rename_fields)
 
         if self.timestamp:
             key = self.timestamp if type(self.timestamp) == str else 'timestamp'
             log_record[key] = datetime.fromtimestamp(record.created, tz=timezone.utc)
 
+        self._perform_rename_log_fields(log_record)
+
+    def _perform_rename_log_fields(self, log_record):
+        for old_field_name, new_field_name in self.rename_fields.items():
+            log_record[new_field_name] = log_record[old_field_name]
+            del log_record[old_field_name]
+
     def process_log_record(self, log_record):
         """
         Override this method to implement custom logic
@@ -204,9 +212,9 @@ class JsonFormatter(logging.Formatter):
         message_dict: Dict[str, Any] = {}
         # FIXME: logging.LogRecord.msg and logging.LogRecord.message in typeshed
         #        are always type of str. We shouldn't need to override that.
-        if isinstance(record.msg, dict):  # type: ignore
-            message_dict = record.msg  # type: ignore
-            record.message = None
+        if isinstance(record.msg, dict):
+            message_dict = record.msg
+            record.message = ""
         else:
             record.message = record.getMessage()
         # only format time if needed
@@ -221,15 +229,10 @@ class JsonFormatter(logging.Formatter):
             message_dict['exc_info'] = record.exc_text
         # Display formatted record of stack frames
         # default format is a string returned from :func:`traceback.print_stack`
-        try:
-            if record.stack_info and not message_dict.get('stack_info'):
-                message_dict['stack_info'] = self.formatStack(record.stack_info)
-        except AttributeError:
-            # Python2.7 doesn't have stack_info.
-            pass
+        if record.stack_info and not message_dict.get('stack_info'):
+            message_dict['stack_info'] = self.formatStack(record.stack_info)
 
-        log_record: Dict[str, Any]
-        log_record = OrderedDict()
+        log_record: Dict[str, Any] = OrderedDict()
         self.add_fields(log_record, record, message_dict)
         log_record = self.process_log_record(log_record)
 
